# Copyright 2025 Cloudification GmbH
#
# Author: cloudification <contact@cloudification.io>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import base64
import dns.exception

from dns import ipv4
from dns import ipv6

from designate.objects import base
from designate.objects import fields
from designate.objects.record import Record
from designate.objects.record import RecordList


def validate_alpn(alpn_string, params):
    """
    h2 means HTTP/2 over TLS, h3 means HTTP/3
    or QUIC (for example, "alpn"="h2").
    """
    valid_values = ("h3", "h2", "h2c", "http/1.1", "http/1.0")
    values = alpn_string.split(",")
    for value in values:
        if value not in valid_values:
            raise ValueError(f"{value} is not supported."
                             f"Supported values {valid_values}")


def validate_ipv4hint(ipv4hint):
    # Should be list of ipv4 comma separated
    for value in ipv4hint.split(","):
        try:
            ipv4.inet_aton(str(value))
        except dns.exception.SyntaxError:
            raise ValueError(f"Value {value} is not ipv4 address")


def validate_ipv6hint(ipv6hint):
    # Should be list of ipv6 comma separated
    for value in ipv6hint.split(","):
        try:
            ipv6.inet_aton(str(value))
        except dns.exception.SyntaxError:
            raise ValueError(f"Value {value} is not ipv6 address")


def validate_ech(ech):
    if "\\" in ech:
        raise ValueError("escape in ECH value")
    try:
        base64.b64decode(ech)
    except Exception:
        raise ValueError("ECH value is not base64 string")


def validate_port(port):
    if not port.isnumeric():
        raise ValueError('Port is not an integer')


def validate_mandatory(mandatory_params, all_params):
    values = mandatory_params.split(",")
    if not set(values).issubset(
            all_params.keys()
    ) or len(values) > len(all_params):
        raise ValueError(
            f"Record should have all mandatory params."
            f"Mandatory params: {values}."
            f"All params: {all_params}")


def validate_doh(doh):
    if not doh.startswith("/"):
        raise ValueError("dohpath must be relative")
    if len(doh) < 7:
        raise ValueError("Minimal valid dohpath length 7")
    if "{?dns}" not in doh:
        raise ValueError("dohpath MUST contain '{?dns}'")


@base.DesignateRegistry.register
class SVCB(Record):
    """
    General-purpose service binding record type
    Defined in: RFC9460
    """

    params = ["alpn", "ipv4hint", "ipv6hint",
              "port", "ech", "no-default-alpn",
              "mandatory", "dohpath"]

    svcb_params = {"alpn": validate_alpn,
                   "ipv4hint": validate_ipv4hint,
                   "ipv6hint": validate_ipv6hint,
                   "port": validate_port,
                   "ech": validate_ech,
                   "mandatory": validate_mandatory,
                   'dohpath': validate_doh,
                   }

    fields = {
        'priority': fields.IntegerFields(minimum=0, maximum=65535),
        'target': fields.DomainField(maxLength=255),
        'alpn': fields.StringFields(maxLength=255, nullable=True),
        'ech': fields.StringFields(maxLength=255, nullable=True),
        'ipv4hint': fields.StringFields(maxLength=255, nullable=True),
        'ipv6hint': fields.StringFields(maxLength=255, nullable=True),
        'port': fields.StringFields(nullable=True, maxLength=10),
        'no_default_alpn': fields.BooleanField(nullable=True, default=False),
        'mandatory': fields.StringFields(nullable=True, maxLength=255),
        'dohpath': fields.StringFields(nullable=True, maxLength=255)
    }

    def from_string(self, value):
        values = value.split(' ')
        priority, target = values[:2]

        def _svcbparams_to_dict(params):
            result = {}
            for param in params:
                if param == 'no-default-alpn':
                    result[param] = True
                    continue
                if "=" not in param:
                    raise ValueError(f"Missed '=' in value {param}")

                svc_param = param.split("=")
                if len(svc_param[1:]) > 1:
                    if param.startswith("ech"):
                        echstr = []
                        for el in svc_param[1:]:
                            if not el:
                                el = "="
                            echstr.append(el)
                        result[svc_param[0]] = "".join(echstr)
                else:
                    result[svc_param[0]] = svc_param[1]
            return result

        svcbparams = _svcbparams_to_dict(values[2:])
        for param_name, param_value in svcbparams.items():
            if param_name not in self.params:
                raise ValueError(f"Unsupported param {param_name}")
            if param_name == 'no-default-alpn':
                if "alpn" in svcbparams.keys():
                    self.no_default_alpn = True
                    continue
                else:
                    raise ValueError("ALPN should be presented"
                                     " if no-default-alpn param exist")
            else:
                if param_name in ("alpn", "mandatory"):
                    self.svcb_params[param_name](param_value, svcbparams)
                else:
                    self.svcb_params[param_name](param_value)
                if param_name == "alpn":
                    self.alpn = "=".join([param_name, param_value])
                elif param_name == "port":
                    self.port = "=".join([param_name, param_value])
                elif param_name == "ipv4hint":
                    self.ipv4hint = "=".join([param_name, param_value])
                elif param_name == "ipv6hint":
                    self.ipv6hint = "=".join([param_name, param_value])
                elif param_name == "mandatory":
                    self.mandatory = "=".join([param_name, param_value])
                elif param_name == "ech":
                    self.ech = "=".join(["ech", param_value])
                elif param_name == 'dohpath':
                    self.dohpath = "=".join([param_name, param_value])

        try:
            self.priority = int(priority)
        except ValueError:
            raise ValueError('Value priority is not an integer')
        self.target = target

    # The record type is defined in the RFC. This will be used when the record
    # is sent by mini-dns.
    RECORD_TYPE = 64


@base.DesignateRegistry.register
class SVCBList(RecordList):

    LIST_ITEM_TYPE = SVCB

    fields = {
        'objects': fields.ListOfObjectsField('SVCB'),
    }



# Copyright (C) 2013 eNovance SAS <licensing@enovance.com>
# Copyright 2014 eBay Inc.
# Copyright 2015 Zetta.IO.
#
# Author: Ron Rickard <rrickard@ebay.com>
# Author: Artom Lifshitz <artom.lifshitz@enovance.com>
# Author: Dag Stenstad <dag@stenstad.net>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import random
import socket
import ssl

from oslo_log import log as logging

from designate.backend import base
from designate import exceptions


LOG = logging.getLogger(__name__)


class NSD4Backend(base.Backend):

    __backend_status__ = 'untested'

    __plugin_name__ = 'nsd4'
    NSDCT_VERSION = 'NSDCT1'

    def __init__(self, target):
        super().__init__(target)

        self.host = self.options.get('host', '127.0.0.1')
        self.port = int(self.options.get('port', 8952))
        self.certfile = self.options.get('certfile',
                                         '/etc/nsd/nsd_control.pem')
        self.keyfile = self.options.get('keyfile',
                                        '/etc/nsd/nsd_control.key')
        self.pattern = self.options.get('pattern', 'slave')

    def _create_ssl_context(self):
        """Create and configure SSL context with backend settings."""
        context = ssl.create_default_context()
        context.load_cert_chain(certfile=self.certfile, keyfile=self.keyfile)

        verify_ssl = (
            self.options.get('verify_ssl', 'true').lower() == 'true'
        )
        check_hostname = (
            self.options.get('check_hostname', 'true').lower() == 'true'
        )
        ca_certs = self.options.get('ca_certs', None)

        context.check_hostname = check_hostname
        if verify_ssl:
            context.verify_mode = ssl.CERT_REQUIRED
            if ca_certs:
                context.load_verify_locations(cafile=ca_certs)
        else:
            context.verify_mode = ssl.CERT_NONE

        return context

    def _command(self, command):
        """Execute a command on the NSD4 control channel."""
        context = self._create_ssl_context()
        sock_addr = (self.host, self.port)

        with (
            socket.create_connection(sock_addr) as raw_sock,
            context.wrap_socket(raw_sock, server_hostname=self.host) as sock,
            sock.makefile(mode='rw') as stream
        ):
            stream.write(f'{self.NSDCT_VERSION} {command}\n')
            stream.flush()
            return stream.read()

    def _execute_nsd4(self, command):
        try:
            LOG.debug('Executing NSD4 control call: %s on %s',
                      command, self.host)
            result = self._command(command)
        except (ssl.SSLError, OSError) as e:
            LOG.debug('NSD4 control call failure: %s' % e)
            raise exceptions.Backend(e)
        if result.rstrip("\n") != 'ok':
            raise exceptions.Backend(result)

    def create_zone(self, context, zone):
        LOG.debug('Create Zone')
        masters = []
        for master in self.masters:
            host = master['host']
            port = master['port']
            masters.append(f'{host} port {port}')

        # Ensure different MiniDNS instances are targeted for AXFRs
        random.shuffle(masters)

        command = 'addzone {} {}'.format(zone['name'], self.pattern)

        try:
            self._execute_nsd4(command)
        except exceptions.Backend as e:
            # If create fails because the zone exists, don't reraise
            if "already exists" not in str(e):
                raise

    def delete_zone(self, context, zone, zone_params=None):
        LOG.debug('Delete Zone')
        command = 'delzone %s' % zone['name']

        try:
            self._execute_nsd4(command)
        except exceptions.Backend as e:
            # If zone is already deleted, don't reraise
            if "not found" not in str(e):
                raise



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import oslotest.base
import requests_mock
from unittest import mock

from designate.backend import impl_pdns4
from designate import context
from designate import exceptions
from designate import objects
from designate.tests import base_fixtures


class PDNS4BackendTestCase(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.stdlog = base_fixtures.StandardLogging()
        self.useFixture(self.stdlog)

        self.context = mock.Mock()
        self.admin_context = mock.Mock()
        mock.patch.object(
            context.DesignateContext, 'get_admin_context',
            return_value=self.admin_context).start()

        self.base_address = 'http://203.0.113.1:8081/api/v1/servers'
        self.zone = objects.Zone(
            id='e2bed4dc-9d01-11e4-89d3-123b93f75cba',
            name='example.com.',
            email='example@example.com',
        )
        self.target = {
            'id': '4588652b-50e7-46b9-b688-a9bad40a873e',
            'type': 'pdns4',
            'masters': [
                {'host': '192.0.2.1', 'port': 53},
                {'host': '192.0.2.2', 'port': 35},
            ],
            'options': [
                {'key': 'api_endpoint', 'value': 'http://203.0.113.1:8081'},
                {'key': 'api_token', 'value': 'api_key'},
                {'key': 'api_ca_cert', 'value': ''}
            ],
        }

        self.backend = impl_pdns4.PDNS4Backend(
            objects.PoolTarget.from_dict(self.target)
        )

    @requests_mock.mock()
    def test_create_zone_success(self, req_mock):
        req_mock.post(
            '%s/localhost/zones' % self.base_address,
        )
        req_mock.get(
            f'{self.base_address}/localhost/zones/{self.zone.name}',
            status_code=404,
        )

        self.backend.create_zone(self.context, self.zone)

        self.assertEqual(
            req_mock.last_request.json(),
            {
                'kind': 'slave',
                'masters': ['192.0.2.1:53', '192.0.2.2:35'],
                'name': 'example.com.',
            }
        )

        self.assertEqual(
            req_mock.last_request.headers.get('X-API-Key'), 'api_key'
        )

    @requests_mock.mock()
    def test_create_zone_ipv6(self, req_mock):
        self.target['masters'] = [
            {'host': '2001:db8::9abc', 'port': 53},
        ]

        self.backend = impl_pdns4.PDNS4Backend(
            objects.PoolTarget.from_dict(self.target)
        )

        req_mock.post(
            '%s/localhost/zones' % self.base_address,
        )
        req_mock.get(
            f'{self.base_address}/localhost/zones/{self.zone.name}',
            status_code=404,
        )

        self.backend.create_zone(self.context, self.zone)

        self.assertEqual(
            req_mock.last_request.json(),
            {
                'kind': 'slave',
                'masters': ['[2001:db8::9abc]:53'],
                'name': 'example.com.',
            }
        )

        self.assertEqual(
            req_mock.last_request.headers.get('X-API-Key'), 'api_key'
        )

    @requests_mock.mock()
    def test_create_zone_hostname(self, req_mock):
        self.target['masters'] = [
            {'host': 'mdns.designate.example.com.', 'port': 53},
        ]

        self.backend = impl_pdns4.PDNS4Backend(
            objects.PoolTarget.from_dict(self.target)
        )

        req_mock.post(
            '%s/localhost/zones' % self.base_address,
        )
        req_mock.get(
            f'{self.base_address}/localhost/zones/{self.zone.name}',
            status_code=404,
        )

        self.backend.create_zone(self.context, self.zone)

        self.assertEqual(
            req_mock.last_request.json(),
            {
                'kind': 'slave',
                'masters': ['mdns.designate.example.com.:53'],
                'name': 'example.com.',
            }
        )

        self.assertEqual(
            req_mock.last_request.headers.get('X-API-Key'), 'api_key'
        )

    @requests_mock.mock()
    def test_create_zone_already_exists(self, req_mock):
        req_mock.post(
            '%s/localhost/zones' % self.base_address,
        )
        req_mock.get(
            f'{self.base_address}/localhost/zones/{self.zone.name}',
            status_code=200,
        )
        req_mock.delete(
            '%s/localhost/zones/example.com.' % self.base_address,
        )

        self.backend.create_zone(self.context, self.zone)

        self.assertEqual(
            req_mock.last_request.json(),
            {
                'kind': 'slave',
                'masters': ['192.0.2.1:53', '192.0.2.2:35'],
                'name': 'example.com.',
            }
        )

        self.assertEqual(
            req_mock.last_request.headers.get('X-API-Key'), 'api_key'
        )

    @requests_mock.mock()
    def test_create_zone_already_exists_and_fails_to_delete(self, req_mock):
        req_mock.post(
            '%s/localhost/zones' % self.base_address,
            status_code=500,
        )
        req_mock.get(
            f'{self.base_address}/localhost/zones/{self.zone.name}',
            status_code=200,
        )
        req_mock.delete(
            '%s/localhost/zones/example.com.' % self.base_address,
            status_code=500,
        )

        self.assertRaisesRegex(
            exceptions.Backend,
            '500 Server Error: None for url: '
            '%s/localhost/zones' % self.base_address,
            self.backend.create_zone, self.context, self.zone
        )

        self.assertIn(
            "Could not delete pre-existing zone "
            "<Zone id:'e2bed4dc-9d01-11e4-89d3-123b93f75cba' "
            "type:'None' name:'example.com.' pool_id:'None' serial:'None' "
            "action:'None' status:'None' shard:'None'>",
            self.stdlog.logger.output
        )

        self.assertIn(
            "<Zone id:'e2bed4dc-9d01-11e4-89d3-123b93f75cba' type:'None' "
            "name:'example.com.' pool_id:'None' serial:'None' action:'None' "
            "status:'None' shard:'None'> exists on the server. "
            "Deleting zone before creation",
            self.stdlog.logger.output
        )

    @requests_mock.mock()
    def test_create_zone_with_tsigkey(self, req_mock):
        req_mock.post(
            '%s/localhost/zones' % self.base_address,
        )
        req_mock.get(
            f'{self.base_address}/localhost/zones/{self.zone.name}',
            status_code=404,
        )

        target = dict(self.target)
        target['options'].append(
            {'key': 'tsigkey_name', 'value': 'tsig_key'}
        )
        backend = impl_pdns4.PDNS4Backend(
            objects.PoolTarget.from_dict(target)
        )

        backend.create_zone(self.context, self.zone)

        self.assertEqual(
            req_mock.last_request.json(),
            {
                'kind': 'slave',
                'masters': ['192.0.2.1:53', '192.0.2.2:35'],
                'name': 'example.com.',
                'slave_tsig_key_ids': ['tsig_key'],
            }
        )

        self.assertEqual(
            req_mock.last_request.headers.get('X-API-Key'), 'api_key'
        )

    @requests_mock.mock()
    def test_create_zone_fail(self, req_mock):
        req_mock.post(
            '%s/localhost/zones' % self.base_address,
            status_code=500,
        )
        req_mock.get(
            f'{self.base_address}/localhost/zones/{self.zone.name}',
            status_code=404,
        )

        self.assertRaisesRegex(
            exceptions.Backend,
            '500 Server Error: None for url: '
            '%s/localhost/zones' % self.base_address,
            self.backend.create_zone, self.context, self.zone
        )

        self.assertEqual(
            req_mock.last_request.headers.get('X-API-Key'), 'api_key'
        )

    @requests_mock.mock()
    def test_create_zone_fail_with_failed_delete(self, req_mock):
        req_mock.post(
            '%s/localhost/zones' % self.base_address,
            status_code=500,
        )
        req_mock.get(
            f'{self.base_address}/localhost/zones/{self.zone.name}',
            [{'status_code': 404}, {'status_code': 200}],
        )
        req_mock.delete(
            '%s/localhost/zones/example.com.' % self.base_address,
            status_code=500,
        )

        self.assertRaisesRegex(
            exceptions.Backend,
            '500 Server Error: None for url: '
            '%s/localhost/zones' % self.base_address,
            self.backend.create_zone, self.context, self.zone
        )

        self.assertEqual(
            req_mock.last_request.headers.get('X-API-Key'), 'api_key'
        )

        self.assertIn(
            "<Zone id:'e2bed4dc-9d01-11e4-89d3-123b93f75cba' type:'None' "
            "name:'example.com.' pool_id:'None' serial:'None' action:'None' "
            "status:'None' shard:'None'> was created with an error. "
            "Deleting zone",
            self.stdlog.logger.output
        )

        self.assertIn(
            "Could not delete errored zone "
            "<Zone id:'e2bed4dc-9d01-11e4-89d3-123b93f75cba' type:'None' "
            "name:'example.com.' pool_id:'None' serial:'None' action:'None' "
            "status:'None' shard:'None'>",
            self.stdlog.logger.output
        )

    @requests_mock.mock()
    def test_delete_zone_success(self, req_mock):
        req_mock.delete(
            '%s/localhost/zones/example.com.' % self.base_address,
        )
        req_mock.get(
            f'{self.base_address}/localhost/zones/{self.zone.name}',
            status_code=200,
        )

        self.backend.delete_zone(self.context, self.zone)

        self.assertEqual(
            req_mock.last_request.headers.get('X-API-Key'), 'api_key'
        )

    @requests_mock.mock()
    def test_delete_zone_missing(self, req_mock):
        req_mock.delete(
            '%s/localhost/zones/example.com.' % self.base_address,
        )

        # pdns returns 422 if asked about a zone that doesn't exist.
        req_mock.get(
            f'{self.base_address}/localhost/zones/{self.zone.name}',
            status_code=422,
        )

        self.backend.delete_zone(self.context, self.zone)

        self.assertEqual(
            req_mock.last_request.headers.get('X-API-Key'), 'api_key'
        )

    @requests_mock.mock()
    def test_delete_zone_fail(self, req_mock):
        req_mock.delete(
            '%s/localhost/zones/example.com.' % self.base_address,
            status_code=500,
        )
        req_mock.get(
            f'{self.base_address}/localhost/zones/{self.zone.name}',
            status_code=200,
        )

        self.assertRaisesRegex(
            exceptions.Backend,
            '500 Server Error: None for url: '
            '%s/localhost/zones' % self.base_address,
            self.backend.delete_zone, self.context, self.zone
        )

        self.assertEqual(
            req_mock.last_request.headers.get('X-API-Key'), 'api_key'
        )

    @mock.patch('os.path.exists')
    def test_verify_ssl(self, mock_path_exists):
        mock_path_exists.return_value = True

        self.backend.api_ca_cert = 'valid_cert'

        self.assertEqual('valid_cert', self.backend._verify_ssl())

    @mock.patch('os.path.exists')
    def test_verify_ssl_does_not_exist(self, mock_path_exists):
        mock_path_exists.return_value = False

        self.backend.api_ca_cert = 'valid_cert'

        self.assertFalse(self.backend._verify_ssl())

    def test_verify_ssl_not_valid(self):
        self.assertFalse(self.backend._verify_ssl())

        self.backend.api_ca_cert = 'changeme'
        self.assertFalse(self.backend._verify_ssl())

        self.backend.api_ca_cert = ''
        self.assertFalse(self.backend._verify_ssl())

        self.backend.api_ca_cert = None
        self.assertFalse(self.backend._verify_ssl())



# Copyright 2025 Cloudification GmbH
#
# Author: cloudification <contact@cloudification.io>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from designate.objects import base
from designate.objects import fields
from designate.objects.record import RecordList
from designate.objects import rrdata_svcb


@base.DesignateRegistry.register
class HTTPS(rrdata_svcb.SVCB):
    """
    SVCB-compatible type for use with HTTPS
    Defined in: RFC9460
    """

    params = ["alpn", "ipv4hint", "ipv6hint",
              "port", "ech", "no-default-alpn",
              "mandatory"]

    svcb_params = {"alpn": rrdata_svcb.validate_alpn,
                   "ipv4hint": rrdata_svcb.validate_ipv4hint,
                   "ipv6hint": rrdata_svcb.validate_ipv6hint,
                   "port": rrdata_svcb.validate_port,
                   "ech": rrdata_svcb.validate_ech,
                   "mandatory": rrdata_svcb.validate_mandatory,
                   }

    fields = {
        'priority': fields.IntegerFields(minimum=0, maximum=65535),
        'target': fields.DomainField(maxLength=255),
        'alpn': fields.StringFields(maxLength=255, nullable=True),
        'ech': fields.StringFields(maxLength=255, nullable=True),
        'ipv4hint': fields.StringFields(maxLength=255, nullable=True),
        'ipv6hint': fields.StringFields(maxLength=255, nullable=True),
        'port': fields.StringFields(nullable=True, maxLength=10),
        'no_default_alpn': fields.BooleanField(nullable=True, default=False),
        'mandatory': fields.StringFields(nullable=True, maxLength=255),
    }

    def from_string(self, value):
        values = value.split(' ')
        priority, target = values[:2]

        def _svcbparams_to_dict(params):
            result = {}
            for param in params:
                if param == 'no-default-alpn':
                    result[param] = True
                    continue
                if "=" not in param:
                    raise ValueError(f"Missed '=' in value {param}")

                svc_param = param.split("=")
                if len(svc_param[1:]) > 1:
                    if param.startswith("ech"):
                        echstr = []
                        for el in svc_param[1:]:
                            if not el:
                                el = "="
                            echstr.append(el)
                        result[svc_param[0]] = "".join(echstr)
                else:
                    result[svc_param[0]] = svc_param[1]
            return result

        svcbparams = _svcbparams_to_dict(values[2:])
        for param_name, param_value in svcbparams.items():
            if param_name not in self.params:
                raise ValueError(f"Unsupported param {param_name}")
            if param_name == 'no-default-alpn':
                if "alpn" in svcbparams.keys():
                    self.no_default_alpn = True
                    continue
                else:
                    raise ValueError("ALPN should be presented"
                                     " if no-default-alpn param exist")
            else:
                if param_name in ("alpn", "mandatory"):
                    self.svcb_params[param_name](param_value, svcbparams)
                else:
                    self.svcb_params[param_name](param_value)
                if param_name == "alpn":
                    self.alpn = "=".join([param_name, param_value])
                elif param_name == "port":
                    self.port = "=".join([param_name, param_value])
                elif param_name == "ipv4hint":
                    self.ipv4hint = "=".join([param_name, param_value])
                elif param_name == "ipv6hint":
                    self.ipv6hint = "=".join([param_name, param_value])
                elif param_name == "mandatory":
                    self.mandatory = "=".join([param_name, param_value])
                elif param_name == "ech":
                    self.ech = "=".join(["ech", param_value])

        try:
            self.priority = int(priority)
        except ValueError:
            raise ValueError('Value priority is not an integer')
        self.target = target

    RECORD_TYPE = 65


@base.DesignateRegistry.register
class HTTPSList(RecordList):

    LIST_ITEM_TYPE = HTTPS

    fields = {
        'objects': fields.ListOfObjectsField('HTTPS'),
    }



# Copyright 2012 Managed I.T.
#
# Author: Kiall Mac Innes <kiall@managedit.ie>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

# Disable eventlet's greendns monkey patching to prevent dnspython
# compatibility issues. Without this, dnspython's zone parsing fails with
# errors like "TypeError: add(): expected an Rdata" due to conflicts between
# eventlet's patched DNS resolver and dnspython's native implementation.
os.environ['EVENTLET_NO_GREENDNS'] = 'yes'

from oslo_service import backend as oslo_service_backend # noqa
try:
    oslo_service_backend.init_backend(
        oslo_service_backend.BackendType.THREADING)
except oslo_service_backend.exceptions.BackendAlreadySelected:
    # Backend already initialized, this is fine
    pass
